# UniMCI (Universal MPI Correctness Interface)
# Copyright (c) 2008-2009, ZIH, TU Dresden, Federal Republic of Germany
#
# See the file LICENSE.txt in the package base directory for details

# @file CreateMPIFunctionHeader.cmake
#       Reads a header with all MPI functions and creates a 
#       header for the pre and post functions for each call. 
#
# @author Tobias Hilbrich
# @date 02.12.2008

#===================================
# Processing Macro 
#===================================
#Macro to process a file with mpi function prototyes
#It creates UNIMCI pre and post functions
MACRO (UNIMCI_GENERATE_MPI infile outfile filename mpi_version)
  FILE (READ ${infile} CONTENT)
  STRING (REPLACE "\n" "<BREAK><BREAK>" CONTENT ${CONTENT})

  #Open and prepare the output file
  FILE (WRITE ${outfile} 
        "/*\n"
        " * UniMCI (Universal MPI Correctness Interface)\n"
        " * Copyright (c) 2008-2009, ZIH, TU Dresden, Federal Republic of Germany\n"
        " *\n"
        " * See the file LICENSE.txt in the package base directory for details\n"
        " */\n\n"  
        "/** \n"
        " * THIS FILE IS AUTOMATICALLY GENERATED,\n"
        " * ALL MODIFICATIONS WILL BE NON-PERMANENT!\n"
        " * \n"
        " * @file ${filename}\n"
        " * @brief Header file with all the UniMCI ${mpi_version} functions.\n"
        " * @details For each MPI function there exists a pre and a\n"
        " * post function that has to be called before and\n"
        " * after the actual MPI call is issued.\n"
        " * \n"
        " * Excluded functions are:\n"
        " *  - double MPI_Wtime(void);\n"
        " *  - double MPI_Wtick(void);\n"
        " *  - int MPI_Pcontrol(const int level, ...);\n"
        " *  - all c2f and f2c converters\n"
        " * \n"
        " * Also see: @ref Example\n"
        " */\n\n"
        "\n"
        "#ifndef __UNIMCI_${mpi_version}_H\n"
        "#define __UNIMCI_${mpi_version}_H\n"
        "\n"
        "#include <mpi.h>\n"
        "\n"
        "#if defined(__cplusplus)\n"
        "extern \"C\" {\n"
        "#endif\n\n")

  #Loop over all the MPI calls
  STRING (REGEX MATCH "(^|<BREAK>)[a-zA-Z0-9]*[ ]MPI[^<>]*($|<BREAK>)" mpicall "${CONTENT}")
  

  WHILE (mpicall MATCHES "MPI" )
    #remove the found string from content
    STRING (REPLACE "${mpicall}" "" CONTENT ${CONTENT})
  
    #Get the regular MPI call name and build Doxygen tags
    STRING (REGEX REPLACE "(^|<BREAK>)[a-zA-Z0-9]*[ ](MPI_[^( ]*).*" "\\2" mpicallname ${mpicall})
    SET (predoxytag 
         "/**\n * @brief UniMCI pre check for ${mpicallname}.\n * @details For details see the MPI standard.\n */\n")
    SET (postdoxytag 
         "/**\n * @brief UniMCI post check for ${mpicallname}.\n * @details For details see the MPI standard.\n */\n")
  
    #replace 'MPI_' by 'UNIMCI_' (for function name)
    STRING (REGEX REPLACE "((^|<BREAK>)[a-zA-Z0-9]*[ ])MPI_" "\\1UNIMCI_check___MPI_" mpicall ${mpicall})
    
    #kill return value
    STRING (REGEX REPLACE "((^|<BREAK>))[a-zA-Z0-9]*[ ]" "\\1void " mpicall ${mpicall})
   
    #Add sourcefile name, linenumber, and eventDescriptor to the end of the file
    IF (mpicall MATCHES "MPI_.*[(]void[)]")
      #special rule for calls with no arguments
      STRING (REGEX REPLACE 
             "void[)][ ]*($|<BREAK>)" 
             "const char* unimci_file, unsigned int unimci_line, unsigned long long unimci_eventDescriptor)\\1" 
             mpicall ${mpicall})
    ELSE (mpicall MATCHES "MPI_.*[(]void[)]")
      #normal rule
      STRING (REGEX REPLACE 
             "[)][ ]*($|<BREAK>)" 
             " ,const char* unimci_file, unsigned int unimci_line, unsigned long long unimci_eventDescriptor)\\1" 
             mpicall ${mpicall})
    ENDIF (mpicall MATCHES "MPI_.*[(]void[)]")
   
    #create two versions, one for pre call and one for post call
    #pre: mpicall, post: postcall
    SET (postcall ${mpicall})
    STRING (REGEX REPLACE "___" "_pre__" mpicall ${mpicall})
    STRING (REGEX REPLACE "___" "_post__" postcall ${postcall})   
  
    #clean the mpicall
    STRING (REPLACE "<BREAK>" "" mpicall ${mpicall})
    STRING (REPLACE "<BREAK>" "" postcall ${postcall})
 
    FILE (APPEND ${outfile} "${predoxytag}${mpicall};\n${postdoxytag}${postcall};\n\n")
 
    #next iteration 
    STRING (REGEX MATCH "(^|<BREAK>)[a-zA-Z0-9]*[ ]MPI[^<>]*($|<BREAK>)" mpicall "${CONTENT}")
  ENDWHILE (mpicall MATCHES "MPI" )
ENDMACRO (UNIMCI_GENERATE_MPI infile outfile filename mpi_version)

#===================================
# Actual script 
#===================================
#Set$process MPI-1 in and out files
SET (INFILE "${PROJECT_SOURCE_DIR}/src/include/mpi-1-functions.h")
SET (OUTFILE "${PROJECT_BINARY_DIR}/generated/include/unimci-mpi-1.h")
UNIMCI_GENERATE_MPI (${INFILE} ${OUTFILE} "unimci-mpi-1.h" "MPI_1")
FILE (APPEND ${OUTFILE} 
      "\n#if defined(__cplusplus)\n"
      "}\n"
      "#endif\n"
      "\n"
      "#endif /*__UNIMCI_MPI-1_H*/\n")

#Set&process MPI-2 in and out files
SET (INFILE "${PROJECT_SOURCE_DIR}/src/include/mpi-2-functions.h")
SET (OUTFILE "${PROJECT_BINARY_DIR}/generated/include/unimci-mpi-2.h")
UNIMCI_GENERATE_MPI (${INFILE} ${OUTFILE} "unimci-mpi-2.h" "MPI_2")
FILE (APPEND ${OUTFILE} 
      "\n#if defined(__cplusplus)\n"
      "}\n"
      "#endif\n"
      "\n"
      "#endif /*__UNIMCI_MPI-2_H*/\n")




