# UniMCI (Universal MPI Correctness Interface)
# Copyright (c) 2008-2009, ZIH, TU Dresden, Federal Republic of Germany
#
# See the file LICENSE.txt in the package base directory for details

# @file FindMARMOT.cmake
#       UniMCI detector for a Marmot installation. 
#
# @author Tobias Hilbrich
# @date 26.11.2008
#
# Stuff to detect:
#  * MARMOT_MPI_C_FLAGS
#  * MARMOT_MPI_CXX_FLAGS
#  * MARMOT_MPI_F77_FLAGS
#  * MARMOT_MPI_F90_FLAGS
#  * MARMOT_MPI_C_COMPILER
#  * MARMOT_MPI_CXX_COMPILER
#  * MARMOT_MPI_F77_COMPILER
#  * MARMOT_MPI_F90_COMPILER
#  * MARMOT_MPI_C_LD_FLAGS
#  * MARMOT_MPI_CXX_LD_FLAGS
#  * MARMOT_MPI_F77_LD_FLAGS
#  * MARMOT_MPI_F90_LD_FLAGS
#  * MARMOT_MPI_C_LIBS
#  * MARMOT_MPI_CXX_LIBS
#  * MARMOT_MPI_F77_LIBS
#  * MARMOT_MPI_F90_LIBS 
#  * MARMOT_VERSION
#  * MARMOT_HAS_FULL_MPI2

#Check algorithm
#(1) search for a MARMOT_HOME
#(2) see whether it is a CMake installation
#(2a) NO CMake installation
## (2a-1) use EXEC_PROGRAM to execute marmotcc, marmotcxx, marmotf77
## (2a-2) extend these such that they return the libs and flags and such when using a special argument
## (2a-3) compilers to use are gained from marmotcc&friends too 
#(2b) YES CMake installation
## (2b-1) use FIND_PACKAGE FindMarmot.cmake from Marmot installation to identify libaries and such
## (2b-2) set all the flags to empty as they will not be set, should work with most windows installations
## (2b-3) compilers to use are the found mpi compilers
#(3) Test the detected stuff for sanity (using little sample - C,C++,Fortran)
#(4) Finish up

#Remove all generated variables, used for quiet runs that fail
MACRO (UNIMCI_MAC_Clean_FindMARMOT)
  SET (MARMOT_HOME CACHE INTERNAL "" FORCE)
  SET (MARMOT_INC_DIR CACHE INTERNAL "" FORCE)
  SET (MARMOT_LIB_DIR CACHE INTERNAL "" FORCE)
  SET (MARMOT_BIN_DIR CACHE INTERNAL "" FORCE)
  SET (MARMOT_SHARE_DIR CACHE INTERNAL "" FORCE)
  
  FOREACH (LANGUAGE ${MARMOT_LANGUAGES})
    SET (MARMOT_${LANGUAGE}_COMPILER_WRAPPER CACHE INTERNAL "" FORCE)
    SET (MARMOT_MPI_${LANGUAGE}_COMPILER CACHE INTERNAL "" FORCE)
    SET (MARMOT_MPI_${LANGUAGE}_FLAGS CACHE INTERNAL "" FORCE)
    SET (MARMOT_MPI_${LANGUAGE}_LD_FLAGS CACHE INTERNAL "" FORCE)
    SET (MARMOT_MPI_${LANGUAGE}_LIBS CACHE INTERNAL "" FORCE)
    SET (MARMOT_MPI_${LANGUAGE}_LIB_DIR_LIST CACHE INTERNAL "" FORCE)
    SET (MARMOT_MPI_${LANGUAGE}_LIB_LIST CACHE INTERNAL "" FORCE)
  ENDFOREACH (LANGUAGE)
ENDMACRO (UNIMCI_MAC_Clean_FindMARMOT)

#Set basic stuff
SET(MARMOT_FOUND FALSE)
SET (MARMOT_LANGUAGES "C;CXX;F77;F90")
SET (MARMOT_WRAPPERS "marmotcc;marmotcxx;marmotf77;marmotf90") #order corresponds to order in MARMOT_LANGUAGES

#===================================
# (1) Search for a MARMOT_HOME
#===================================
#Do a straight path search
FIND_PATH(MARMOT_HOME include/marmot.h 
          "$ENV{MARMOT_HOME}"
          /
          /usr 
          /usr/local 
          /usr/local/marmot
          /opt
          /opt/marmot
          "$ENV{EXTERNLIBS}/marmot"
          "$ENV{ProgramFiles}/marmot"
          "C:/Program Files/marmot"
          [HKEY_LOCAL_MACHINE\\SOFTWARE\\HLRS\\Marmot]
          DOC "The path for the Marmot installation."
)

#Put all the Marmot pathes into the cache to allow specific values for these (advanced, for weird installations)
MARK_AS_ADVANCED(FORCE MARMOT_INC_DIR MARMOT_LIB_DIR MARMOT_BIN_DIR MARMOT_SHARE_DIR
                       MARMOT_C_COMPILER_WRAPPER MARMOT_CXX_COMPILER_WRAPPER
                       MARMOT_F77_COMPILER_WRAPPER MARMOT_F90_COMPILER_WRAPPER)

# If not found, try to look for "marmotcc" in the PATH (non Windows)
IF (NOT EXISTS "${MARMOT_HOME}")
  FIND_PROGRAM (MARMOT_C_COMPILER_WRAPPER_TEMP marmotcc DOC "Path to the marmot C compiler wrapper, used to automatically detect the marmot home path.")
  SET (MARMOT_C_COMPILER_WRAPPER ${MARMOT_C_COMPILER_WRAPPER_TEMP} CACHE FILEPATH "The full path of the marmot C compiler wrapper. Usually '\${MARMOT_HOME}/bin/marmotcc'" FORCE)
  SET (MARMOT_C_COMPILER_WRAPPER_TEMP "" CACHE INTERNAL "" FORCE)
  MARK_AS_ADVANCED(FORCE MARMOT_C_COMPILER_WRAPPER) 
  IF (MARMOT_C_COMPILER_WRAPPER)
    STRING (REGEX REPLACE "/bin/marmotcc$" "" MARMOT_HOME_TEMP ${MARMOT_C_COMPILER_WRAPPER})
    SET (MARMOT_HOME "${MARMOT_HOME_TEMP}" CACHE PATH "The path for the Marmot installation." FORCE)
  ENDIF (MARMOT_C_COMPILER_WRAPPER)
ENDIF (NOT EXISTS "${MARMOT_HOME}")

#See whether we found a home and check it for sanity (if it was specified by the user)
IF (NOT MARMOT_HOME)
  # Could not find the Marmot home directory, aborting (if not quiet) !
  IF (MARMOT_FIND_REQUIRED)
    SET (MARMOT_HOME "${MARMOT_HOME}" CACHE PATH "The path for the Marmot installation." FORCE) 
    MESSAGE(FATAL_ERROR "Could not find the home directory of your Marmot installation, \nplease specify it in 'MARMOT_HOME' or use another MPI checker!")
  ENDIF (MARMOT_FIND_REQUIRED)
ELSE (NOT MARMOT_HOME)
  # it's defined. check whether path exists
  IF (EXISTS "${MARMOT_HOME}")
    # reset include and library variables
  ELSE (EXISTS "${MARMOT_HOME}")
    SET (MARMOT_HOME "${MARMOT_HOME}" CACHE PATH "The path for the Marmot installation." FORCE)
    MESSAGE(FATAL_ERROR "Given path '${MARMOT_HOME}' for MARMOT_HOME does not exists!")
  ENDIF (EXISTS "${MARMOT_HOME}")
ENDIF (NOT MARMOT_HOME)

#Only continue from here if we found a valid path for Marmot Home
IF (NOT ${MARMOT_HOME} STREQUAL MARMOT_HOME-NOTFOUND)
  #Set all the pathes for include, lib, bin, and share
  IF (NOT MARMOT_INC_DIR)
    SET (MARMOT_INC_DIR "${MARMOT_HOME}/include" CACHE PATH "The Marmot include directory, usually '\${MARMOT_HOME}/include'" FORCE)
  ENDIF (NOT MARMOT_INC_DIR)
  IF ("${MARMOT_INC_DIR}" STREQUAL "${MARMOT_HOME_COPY}/include")
    SET (MARMOT_INC_DIR "${MARMOT_HOME}/include" CACHE PATH "The Marmot include directory, usually '\${MARMOT_HOME}/include'" FORCE)
  ENDIF ("${MARMOT_INC_DIR}" STREQUAL "${MARMOT_HOME_COPY}/include")
  
  IF (NOT MARMOT_LIB_DIR)
    SET (MARMOT_LIB_DIR "${MARMOT_HOME}/lib" CACHE PATH "The Marmot library directory, usually '\${MARMOT_HOME}/lib'" FORCE)
  ENDIF (NOT MARMOT_LIB_DIR)
  IF ("${MARMOT_LIB_DIR}" STREQUAL "${MARMOT_HOME_COPY}/lib")
    SET (MARMOT_LIB_DIR "${MARMOT_HOME}/lib" CACHE PATH "The Marmot library directory, usually '\${MARMOT_HOME}/lib'" FORCE)
  ENDIF ("${MARMOT_LIB_DIR}" STREQUAL "${MARMOT_HOME_COPY}/lib")
  
  IF (NOT MARMOT_BIN_DIR) 
    SET (MARMOT_BIN_DIR "${MARMOT_HOME}/bin" CACHE PATH "The Marmot binary directory, usually '\${MARMOT_HOME}/bin'" FORCE)
  ENDIF (NOT MARMOT_BIN_DIR)
  IF ("${MARMOT_BIN_DIR}" STREQUAL "${MARMOT_HOME_COPY}/bin")
    SET (MARMOT_BIN_DIR "${MARMOT_HOME}/bin" CACHE PATH "The Marmot binary directory, usually '\${MARMOT_HOME}/bin'" FORCE)
  ENDIF ("${MARMOT_BIN_DIR}" STREQUAL "${MARMOT_HOME_COPY}/bin")
  
  IF (NOT MARMOT_SHARE_DIR)
    SET (MARMOT_SHARE_DIR "${MARMOT_HOME}/share" CACHE PATH "The Marmot share directory, usually '\${MARMOT_HOME}/share'" FORCE)
  ENDIF (NOT MARMOT_SHARE_DIR)
  IF ("${MARMOT_SHARE_DIR}" STREQUAL "${MARMOT_HOME_COPY}/share")
    SET (MARMOT_SHARE_DIR "${MARMOT_HOME}/share" CACHE PATH "The Marmot share directory, usually '\${MARMOT_HOME}/share'" FORCE)
  ENDIF ("${MARMOT_SHARE_DIR}" STREQUAL "${MARMOT_HOME_COPY}/share")
  
  #Set the pathes for the marmot compiler wrappers
  FOREACH (LANGUAGE ${MARMOT_LANGUAGES})
      #Get the lower name version of the language and its associated wrapper
      STRING (TOLOWER ${LANGUAGE} LOWER_LANGUAGE)
      UNIMCI_MAC_STRING_CORRESPOND (${LANGUAGE} "${MARMOT_LANGUAGES}" "${MARMOT_WRAPPERS}" LANGUAGE_WRAPPER)
      
      #Set the path, if not speciefied by user
      IF (NOT MARMOT_${LANGUAGE}_COMPILER_WRAPPER)
        SET (MARMOT_${LANGUAGE}_COMPILER_WRAPPER "${MARMOT_BIN_DIR}/${LANGUAGE_WRAPPER}" CACHE FILEPATH "The full path of the marmot ${LANGUAGE} compiler wrapper. Usually \${MARMOT_HOME}/bin/${LANGUAGE_WRAPPER}'" FORCE)
      ENDIF (NOT MARMOT_${LANGUAGE}_COMPILER_WRAPPER)
      IF ("${MARMOT_${LANGUAGE}_COMPILER_WRAPPER}" STREQUAL "${MARMOT_BIN_DIR_COPY}/${LANGUAGE_WRAPPER}")
        SET (MARMOT_${LANGUAGE}_COMPILER_WRAPPER "${MARMOT_BIN_DIR}/${LANGUAGE_WRAPPER}" CACHE FILEPATH "The full path of the marmot ${LANGUAGE} compiler wrapper. Usually \${MARMOT_HOME}/bin/${LANGUAGE_WRAPPER}'" FORCE)
      ENDIF ("${MARMOT_${LANGUAGE}_COMPILER_WRAPPER}" STREQUAL "${MARMOT_BIN_DIR_COPY}/${LANGUAGE_WRAPPER}")
  ENDFOREACH (LANGUAGE)  
ENDIF (NOT ${MARMOT_HOME} STREQUAL MARMOT_HOME-NOTFOUND)

#Make a copy of the MARMOT_HOME variable to allow comparisons to its old value
## Rule: If MARMOT_HOME changed, we should reset the MARMOT_[XXX]_DIR's 
##       Unless the user speciefied them, as we want to adhere to 
##       "Never overwritte a user written variable".
## Same story for the compiler wrappers.
SET(MARMOT_HOME_COPY ${MARMOT_HOME} CACHE INTERNAL "Internal: used to keep track of whether user specified MARMOT_INC_DIR,... or not." FORCE) 
SET(MARMOT_BIN_DIR_COPY ${MARMOT_BIN_DIR} CACHE INTERNAL "Internal: used to keep track of whether user specified MARMOT_C_COMPILER_WRAPPER,... or not." FORCE)

#===================================
# (2) Determine whether it is a CMake installation
#===================================
IF (WIN32)
  SET (MARMOT_INSTALL_TYPE "WINDOWS")
ELSE (WIN32)
  SET (MARMOT_INSTALL_TYPE "WRAPPERS")
ENDIF (WIN32)

#===================================
# (2a/b) Branch based on the result 
#===================================
#Mark all the variables that we need as advanced ...
FOREACH (LANGUAGE ${MARMOT_LANGUAGES})
    MARK_AS_ADVANCED (FORCE 
      MARMOT_MPI_${LANGUAGE}_COMPILER 
      MARMOT_MPI_${LANGUAGE}_FLAGS 
      MARMOT_MPI_${LANGUAGE}_LD_FLAGS 
      MARMOT_MPI_${LANGUAGE}_LIBS
      MARMOT_MPI_${LANGUAGE}_LIB_DIR_LIST
      MARMOT_MPI_${LANGUAGE}_LIB_LIST
      )
ENDFOREACH (LANGUAGE)

##############
#CMAKE
IF (MARMOT_INSTALL_TYPE STREQUAL "WINDOWS")
    #Add the FindMarmot Module from Marmot to the module path
    #SET(Marmot_DIR "${MARMOT_SHARE_DIR}/marmot/cmakemodules")
    SET(CMAKE_MODULE_PATH "${MARMOT_SHARE_DIR}/marmot/cmakemodules;${CMAKE_MODULE_PATH}")
  
    #IF (MARMOT_FIND_QUIETLY)
    #  FIND_PACKAGE(Marmot QUIET)
    #ENDIF (MARMOT_FIND_QUIETLY)
    #
    #IF (MARMOT_FIND_REQUIRED)
    #  FIND_PACKAGE(Marmot REQUIRED)
    #ENDIF (MARMOT_FIND_REQUIRED)
    #
    #MESSAGE (FATAL_ERROR "The 'FindMarmot.cmake' provided by Marmots CMake installation does not support what we need ! \n We have to take care that we change this, currently this method is unsupported.")
    
    #Hardcoded results for MPICH and Marmot DEBUG !
    SET (MPICH_INSTALL_DIR "C:/Programme/MPICH2")
    FOREACH (LANGUAGE ${MARMOT_LANGUAGES})
        #Get a lower case version of the language name
        STRING (TOLOWER ${LANGUAGE} LOWER_LANGUAGE)
          
        #Compiler
        SET (MARMOT_MPI_${LANGUAGE}_COMPILER ${CMAKE_${LANGUAGE}_COMPILER} CACHE FILEPATH "Path for the ${LANGUAGE} compiler used by Marmot for MPI." FORCE)
        #Flags
        SET (MARMOT_MPI_${LANGUAGE}_FLAGS "/I\"${MARMOT_INC_DIR}\" /I\"${MPICH_INSTALL_DIR}/include\" /DMARMOT_NO_MPI_ENHANCE" CACHE STRING "${LANGUAGE} flags used by Marmot for MPI." FORCE)
        #ld-flags
        SET (MARMOT_MPI_${LANGUAGE}_LD_FLAGS "" CACHE STRING "The linker flags (for ${LANGUAGE}) used by Marmot for MPI." FORCE)
        #Libs
        SET (MARMOT_MPI_${LANGUAGE}_LIBS "" CACHE STRING "The libraries and paths (for C) used by Marmot for MPI." FORCE)
        #Version
        SET (MARMOT_VERSION "2.1.0" CACHE INTERNAL "The Marmot version." FORCE)
                        
        #Set libs and lib dirs
        SET (MARMOT_MPI_${LANGUAGE}_LIB_DIR_LIST "${MPICH_INSTALL_DIR}/lib;${MARMOT_LIB_DIR}" CACHE INTERNAL "Generated library dir list" FORCE)
        SET (MARMOT_MPI_${LANGUAGE}_LIB_LIST "marmot-profiled.lib;marmot-cored.lib;mpi.lib" CACHE INTERNAL "Generated library list" FORCE)
    ENDFOREACH (LANGUAGE)
        
    
ENDIF (MARMOT_INSTALL_TYPE STREQUAL "WINDOWS")

##############
#AUTOTOOLS
IF (MARMOT_INSTALL_TYPE STREQUAL "WRAPPERS")

    FOREACH (LANGUAGE ${MARMOT_LANGUAGES})
      # See which compiler wrappers are available and ask them for infos
##Removed: if no full pathes given it does not works as expected ...
#      IF (EXISTS ${MARMOT_${LANGUAGE}_COMPILER_WRAPPER})
        #Only query the pathes, libs and flags if the compiler wrapper changed
        IF (NOT MARMOT_${LANGUAGE}_COMPILER_WRAPPER STREQUAL MARMOT_${LANGUAGE}_COMPILER_WRAPPER_COPY)
          #Get a lower case version of the language name
          STRING (TOLOWER ${LANGUAGE} LOWER_LANGUAGE)
          
          #See whether the wrapper works
          EXEC_PROGRAM (${MARMOT_${LANGUAGE}_COMPILER_WRAPPER} ./ ARGS --help OUTPUT_VARIABLE TEMP RETURN_VALUE RETURN)
          
          #if a --help works we assume the wrapper is there and works fine
          IF (NOT RETURN)
            #Compiler
            EXEC_PROGRAM (${MARMOT_${LANGUAGE}_COMPILER_WRAPPER} ./ ARGS --marmot-config-get-backend-compiler OUTPUT_VARIABLE MARMOT_MPI_${LANGUAGE}_COMPILER)
            SET (MARMOT_MPI_${LANGUAGE}_COMPILER ${MARMOT_MPI_${LANGUAGE}_COMPILER} CACHE FILEPATH "Path for the ${LANGUAGE} compiler used by Marmot for MPI." FORCE)
            #Flags
            EXEC_PROGRAM (${MARMOT_${LANGUAGE}_COMPILER_WRAPPER} ./ ARGS --marmot-config-get-${LOWER_LANGUAGE}-flags OUTPUT_VARIABLE MARMOT_MPI_${LANGUAGE}_FLAGS)
            SET (MARMOT_MPI_${LANGUAGE}_FLAGS "${MARMOT_MPI_${LANGUAGE}_FLAGS} -DMARMOT_NO_MPI_ENHANCE" CACHE STRING "${LANGUAGE} flags used by Marmot for MPI." FORCE)
            #ld-flags
            EXEC_PROGRAM (${MARMOT_${LANGUAGE}_COMPILER_WRAPPER} ./ ARGS --marmot-config-get-ld-flags OUTPUT_VARIABLE MARMOT_MPI_${LANGUAGE}_LD_FLAGS)
            STRING (REPLACE "\n" "" MARMOT_MPI_${LANGUAGE}_LD_FLAGS ${MARMOT_MPI_${LANGUAGE}_LD_FLAGS})
            SET (MARMOT_MPI_${LANGUAGE}_LD_FLAGS ${MARMOT_MPI_${LANGUAGE}_LD_FLAGS} CACHE STRING "The linker flags (for ${LANGUAGE}) used by Marmot for MPI." FORCE)
            #Libs
            EXEC_PROGRAM (${MARMOT_${LANGUAGE}_COMPILER_WRAPPER} ./ ARGS --marmot-config-get-libs OUTPUT_VARIABLE MARMOT_MPI_${LANGUAGE}_LIBS)
            SET (MARMOT_MPI_${LANGUAGE}_LIBS ${MARMOT_MPI_${LANGUAGE}_LIBS} CACHE STRING "The libraries and paths (for C) used by Marmot for MPI." FORCE)
            #Version
            EXEC_PROGRAM (${MARMOT_${LANGUAGE}_COMPILER_WRAPPER} ./ ARGS --version OUTPUT_VARIABLE MARMOT_VERSION_TEMP)
            SET (MARMOT_VERSION ${MARMOT_VERSION_TEMP} CACHE INTERNAL "The Marmot version." FORCE)
            #Has MPI2
            UNIMCI_MAC_MARMOT_VERSION_GE ("${MARMOT_VERSION}" "2.4.0" result)
            IF (result)
                #Marmot 2.4.0 and above have the "--marmot-has-mpi2" flag in marmotcc
                EXEC_PROGRAM (${MARMOT_C_COMPILER_WRAPPER} ./ ARGS --marmot-has-mpi2 OUTPUT_VARIABLE MARMOT_HAS_MPI_TEMP)
                IF (${MARMOT_HAS_MPI_TEMP} MATCHES "YES")
                    SET (MARMOT_HAS_FULL_MPI2 TRUE CACHE INTERNAL "Has the Marmot full MPI2 support ?" FORCE)
                ELSE (${MARMOT_HAS_MPI_TEMP} MATCHES "YES")
                    SET (MARMOT_HAS_FULL_MPI2 FALSE CACHE INTERNAL "Has the Marmot full MPI2 support ?" FORCE)
                ENDIF (${MARMOT_HAS_MPI_TEMP} MATCHES "YES")
            ELSE (result)
                SET (MARMOT_HAS_FULL_MPI2 FALSE CACHE INTERNAL "Has the Marmot full MPI2 support ?" FORCE)
            ENDIF (result)
                        
            #Parse the stuff returned by libs
            UNIMCI_MAC_GET_LIB_DIR_LIST ("${MARMOT_MPI_${LANGUAGE}_LIBS}" MARMOT_MPI_${LANGUAGE}_LIB_DIR_LIST)
            UNIMCI_MAC_GET_LIB_LIST ("${MARMOT_MPI_${LANGUAGE}_LIBS}" MARMOT_MPI_${LANGUAGE}_LIB_LIST)
            SET (MARMOT_MPI_${LANGUAGE}_LIB_DIR_LIST ${MARMOT_MPI_${LANGUAGE}_LIB_DIR_LIST} CACHE INTERNAL "Generated library dir list" FORCE)
            SET (MARMOT_MPI_${LANGUAGE}_LIB_LIST ${MARMOT_MPI_${LANGUAGE}_LIB_LIST} CACHE INTERNAL "Generated library list" FORCE)
          ENDIF (NOT RETURN)
        ENDIF (NOT MARMOT_${LANGUAGE}_COMPILER_WRAPPER STREQUAL MARMOT_${LANGUAGE}_COMPILER_WRAPPER_COPY)
#      ENDIF (EXISTS ${MARMOT_${LANGUAGE}_COMPILER_WRAPPER})
    ENDFOREACH (LANGUAGE)
    
    ##Make a copy of each compiler wrapper variable.
    ##Used to keep track of whether a compiler wrapper changed or not.
    ##If it did not change since last run, we will not change the above pathes, libs and dirs.
    ##If so, we will change them.
    FOREACH (LANGUAGE ${MARMOT_LANGUAGES})
      SET(MARMOT_${LANGUAGE}_COMPILER_WRAPPER_COPY ${MARMOT_${LANGUAGE}_COMPILER_WRAPPER} CACHE INTERNAL "Internal: used to keep track of whether user overwrote the flags returned by the compiler wrappers or not." FORCE)
    ENDFOREACH (LANGUAGE)    
    
ENDIF (MARMOT_INSTALL_TYPE STREQUAL "WRAPPERS")

#===================================
# (3) Test the detected stuff
#===================================
#To test it we see whether we can at least compile and link a C application with Marmot
#Create a CMakeLists.txt file which will generate the test

IF (WIN32)
    STRING (REPLACE "\"" "\\\"" MARMOT_MPI_C_FLAGS_TEMP ${MARMOT_MPI_C_FLAGS})
ELSE (WIN32)
    SET  (MARMOT_MPI_C_FLAGS_TEMP "${MARMOT_MPI_C_FLAGS}")    
ENDIF (WIN32)

FILE(WRITE ${PROJECT_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/CMakeTmp/CMakeLists.txt
  "CMAKE_MINIMUM_REQUIRED(VERSION 2.6)\n"
  "PROJECT(mpitest C)\n"
  "SET(CMAKE_VERBOSE_MAKEFILE \"TRUE\")\n"
  "SET(CMAKE_C_FLAGS \"${MARMOT_MPI_C_FLAGS_TEMP}\")\n"
  "SET(CMAKE_C_COMPILER \"${MARMOT_MPI_C_COMPILER}\")\n"
  "SET(CMAKE_EXE_LINKER_FLAGS \"${MARMOT_MPI_LD_FLAGS}\")\n"
  "LINK_DIRECTORIES("
  )

FOREACH (LIB ${MARMOT_MPI_C_LIB_DIR_LIST})
   FILE (APPEND ${PROJECT_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/CMakeTmp/CMakeLists.txt
         "\"${LIB}\" ")
ENDFOREACH (LIB)

FILE (APPEND ${PROJECT_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/CMakeTmp/CMakeLists.txt
  ")\n"
  "LINK_LIBRARIES (${MARMOT_MPI_C_LIB_LIST})\n"
  "ADD_EXECUTABLE(mpitest mpitest.c)\n"
  )
                
# Create a simple MPI application
FILE(WRITE ${PROJECT_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/CMakeTmp/mpitest.c
  "        #include <mpi.h>\n"
  "        int main (int argc, char** argv)\n"
  "        {\n"
  "          MPI_Init (&argc,&argv);\n"
  "          MPI_Finalize ();\n"
  "          return 0;\n"
  "        }\n"
  "\n" #the C standard required newline ...
  )
                
# Use TRY_COMPILE to make the target "flib"
TRY_COMPILE(
  FTEST_OK
  ${PROJECT_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/CMakeTmp
  ${PROJECT_BINARY_DIR}${CMAKE_FILES_DIRECTORY}/CMakeTmp
  mpitest
  OUTPUT_VARIABLE MY_OUTPUT)

IF (FTEST_OK)
  #The test succeded, so we have a Marmot !!!
  SET(MARMOT_FOUND TRUE)
  MESSAGE (STATUS "Marmot configuration works for C.")
ELSE (FTEST_OK)
  #See whether we should tell something to the user ?
  IF (MARMOT_FIND_REQUIRED)
    MESSAGE (FATAL_ERROR "Compilation of a simple MPI application with Marmot failed. \n The error output is:\n\n${MY_OUTPUT}")
  ENDIF (MARMOT_FIND_REQUIRED)
ENDIF (FTEST_OK)
 
#===================================
# (4) Finish up
#===================================
# Do we have to give some output ?
IF (MARMOT_FOUND)
   IF (NOT MARMOT_FIND_QUIETLY)
      MESSAGE(STATUS "Found MARMOT")
   ENDIF (NOT MARMOT_FIND_QUIETLY)
   
   #Set the UNIMCI Variables
   #USED_MPI_CHECKER will already be set
   FOREACH (LANGUAGE ${MARMOT_LANGUAGES})
    #Only copy the compiler name if it actually exists ... 
    IF (EXISTS "${MARMOT_MPI_${LANGUAGE}_COMPILER}")    
        SET (UNIMCI_MPI_${LANGUAGE}_COMPILER ${MARMOT_MPI_${LANGUAGE}_COMPILER} CACHE FILEPATH "Compiler used for MPI and ${LANGUAGE}.")
    ELSE (EXISTS "${MARMOT_MPI_${LANGUAGE}_COMPILER}")
        FIND_PROGRAM (UNIMCI_MPI_${LANGUAGE}_COMPILER_TEMP "${MARMOT_MPI_${LANGUAGE}_COMPILER}")
        IF (NOT UNIMCI_MPI_${LANGUAGE}_COMPILER)
          SET (UNIMCI_MPI_${LANGUAGE}_COMPILER "" CACHE FILEPATH "Compiler used for MPI and ${LANGUAGE}.")
        ELSE (NOT UNIMCI_MPI_${LANGUAGE}_COMPILER)
          SET (UNIMCI_MPI_${LANGUAGE}_COMPILER_TEMP ${MARMOT_MPI_${LANGUAGE}_COMPILER} CACHE FILEPATH "Compiler used for MPI and ${LANGUAGE}.")
        ENDIF (NOT UNIMCI_MPI_${LANGUAGE}_COMPILER)
        #get rid of the temp variable ...
        SET (UNIMCI_MPI_${LANGUAGE}_COMPILER_TEMP "" CACHE INTERNAL "" FORCE)
    ENDIF (EXISTS "${MARMOT_MPI_${LANGUAGE}_COMPILER}")
    
    SET (UNIMCI_MPI_${LANGUAGE}_FLAGS "${MARMOT_MPI_${LANGUAGE}_FLAGS}" CACHE STRING "Flags used for MPI and ${LANGUAGE}.")
    SET (UNIMCI_MPI_${LANGUAGE}_LD_FLAGS ${MARMOT_MPI_${LANGUAGE}_LD_FLAGS} CACHE STRING "Linker flags used for MPI and ${LANGUAGE}.")
    SET (UNIMCI_MPI_${LANGUAGE}_LIB_DIR_LIST ${MARMOT_MPI_${LANGUAGE}_LIB_DIR_LIST} CACHE STRING "Library directories used for MPI and ${LANGUAGE} (semicolon seperated list).")
    
    IF (MARMOT_MPI_${LANGUAGE}_LIB_LIST)
        SET (MARMOT_MPI_${LANGUAGE}_LIB_LIST_COPY ${MARMOT_MPI_${LANGUAGE}_LIB_LIST})
        #TODO think about all possible version that we might get here
        SET (MARMOT_MPI_WRAPPERS "-lmarmot-profile" "-lmarmot-profiled" "-lmarmot-profiler"
                                 "-libmarmot-profiled" "-libmarmot-profiler" "-libmarmot-profile" 
                                 "marmot-profiled.lib" "marmot-profiler.lib" "marmot-profile.lib"
                                 "libmarmot-profiled" "libmarmot-profiler" "libmarmot-profile"
                                 "marmot-profiled" "marmot-profiled" "marmot-profile"
                                 "-lmarmot-fortran" "-lmarmot-fortrand" "-lmarmot-fortranr" 
                                 "-libmarmot-fortrand" "-libmarmot-fortranr" "-libmarmot-fortran"
                                 "marmot-fortrand.lib" "marmot-fortranr.lib" "marmot-fortran.lib" 
                                 "libmarmot-fortrand" "libmarmot-fortranr" "libmarmot-fortran" 
                                 "marmot-fortrand" "marmot-fortrand" "marmot-fortran")
        FOREACH (WRAPPER_LIB ${MARMOT_MPI_WRAPPERS}) 
          LIST (REMOVE_ITEM MARMOT_MPI_${LANGUAGE}_LIB_LIST_COPY ${WRAPPER_LIB})
        ENDFOREACH (WRAPPER_LIB)
        
        #Add the unimci lib
        #To avoid the libname mess I just search for marmot-core and do some replace magic
        SET (UNIMCI_MARMOT_LIB "")
        FOREACH (MARMOT_LIB ${MARMOT_MPI_${LANGUAGE}_LIB_LIST_COPY})
          IF (${MARMOT_LIB} MATCHES "marmot-core")
             STRING (REGEX REPLACE "marmot-core" "marmot-unimci" UNIMCI_MARMOT_LIB ${MARMOT_LIB}) 
          ENDIF (${MARMOT_LIB} MATCHES "marmot-core")
        ENDFOREACH (MARMOT_LIB)
        SET (MARMOT_MPI_${LANGUAGE}_LIB_LIST_COPY ${UNIMCI_MARMOT_LIB} ${MARMOT_MPI_${LANGUAGE}_LIB_LIST_COPY})
        
        #Set the final cache variable
        SET (UNIMCI_MPI_${LANGUAGE}_LIB_LIST ${MARMOT_MPI_${LANGUAGE}_LIB_LIST_COPY} CACHE STRING "Libraries used for MPI and ${LANGUAGE} (semicolon seperated list).")
    ENDIF (MARMOT_MPI_${LANGUAGE}_LIB_LIST)
   ENDFOREACH (LANGUAGE)
   
   #Copy the Marmot version to Checker version
   SET (CHECKER_VERSION ${MARMOT_VERSION} CACHE INTERNAL "The version of the used MPI checker." FORCE)
   
ELSE (MARMOT_FOUND)
   #If we searched quietly, remove ALL the variables that we generated in this find module!
   IF (MARMOT_FIND_QUIETLY)
     UNIMCI_MAC_Clean_FindMARMOT ()
   ENDIF (MARMOT_FIND_QUIETLY)

   IF (MARMOT_FIND_REQUIRED)
      MESSAGE(FATAL_ERROR "Could not find Marmot")
   ENDIF (MARMOT_FIND_REQUIRED)
ENDIF (MARMOT_FOUND)
