/*
 * UniMCI (Universal MPI Correctness Interface)
 * Copyright (c) 2008-2009, ZIH, TU Dresden, Federal Republic of Germany
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file utility.c source for the UniMCI utility library.
 *
 * This source implements utility functions that are
 * provided by UniMCI, e.g., messages.
 *
 * @author Tobias Hilbrich
 * @date 03.12.08
 */

#include "unimci-message.h"
#include <malloc.h>
#include <string.h>
#include <assert.h>
#include <stdio.h>

/*========================================*/
/*          UNIMCI_msg_free               */
/*========================================*/
int UNIMCI_msg_free (UNIMCI_MSG **msg)
{
    if (*msg)
    {
        if ((*msg)->strCallInfo)
            free ((*msg)->strCallInfo);
        (*msg)->strCallInfo = NULL;

        if ((*msg)->strText)
            free ((*msg)->strText);
        (*msg)->strText = NULL;

        if ((*msg)->strHTMLReference)
            free ((*msg)->strHTMLReference);
        (*msg)->strHTMLReference = NULL;

        if ((*msg)->isGlobal && (*msg)->participatingRanks)
            free ((*msg)->participatingRanks);
        (*msg)->participatingRanks = NULL;

        free (*msg);
    }

    *msg = NULL;

    return 0;
}

/*========================================*/
/*          UNIMCI_msg_new                */
/*========================================*/
int UNIMCI_msg_new (UNIMCI_MSG **msg)
{
    UNIMCI_MSG* p;
    p = (UNIMCI_MSG*) malloc (sizeof (UNIMCI_MSG));
    assert (p);

    p->msgType = UNIMCI_MSG_TYPE_INFO;

    p->strCallInfo = (char*) malloc (sizeof (char)*1);
    assert (p->strCallInfo);
    sprintf (p->strCallInfo,"");

    p->strText = (char*) malloc (sizeof (char)*1);
    assert (p->strText);
    sprintf (p->strText,"");

    p->strHTMLReference = (char*) malloc (sizeof (char)*1);
    assert (p->strHTMLReference);
    sprintf (p->strHTMLReference,"");

    p->isGlobal = 0;
    p->numParticipatingRanks = 0;
    p->participatingRanks = NULL;

    p->eventDescriptor = 0;

    *msg = p;

    return 0;
}

/*========================================*/
/*          UNIMCI_msg_clone              */
/*========================================*/
int UNIMCI_msg_clone (UNIMCI_MSG *msg_in, UNIMCI_MSG **msg_out)
{
    UNIMCI_MSG *p;
    int i;

    p = (UNIMCI_MSG*) malloc (sizeof (UNIMCI_MSG));

    p->msgType = msg_in->msgType;

    p->strCallInfo = NULL;
    UNIMCI_msg_set_callinfo (p, msg_in->strCallInfo);
    p->strText = NULL;
    UNIMCI_msg_set_text (p, msg_in->strText);
    p->strHTMLReference = NULL;
    UNIMCI_msg_set_reference (p, msg_in->strHTMLReference);

    p->isGlobal = msg_in->isGlobal;
    p->participatingRanks = NULL;
    if (p->isGlobal)
    {
        UNIMCI_msg_set_global (p,msg_in->numParticipatingRanks,msg_in->participatingRanks);
    }
    else
    {
        p->numParticipatingRanks = 0;
    }

    p->eventDescriptor = msg_in->eventDescriptor;

    return 0;
}

/*========================================*/
/*          UNIMCI_msg_set_text           */
/*========================================*/
int UNIMCI_msg_set_text (UNIMCI_MSG *msg_inout ,const char* strTextToSet)
{
    int len;

    if (msg_inout->strText)
        free (msg_inout->strText);

    len = strlen (strTextToSet);
    msg_inout->strText = (char*) malloc (sizeof(char) * (len+1));
    assert (msg_inout->strText);
    if (strTextToSet)
        sprintf (msg_inout->strText, "%s", strTextToSet);
    else
        sprintf (msg_inout->strText, "");

    return 0;
}

/*========================================*/
/*          UNIMCI_msg_set_reference      */
/*========================================*/
int UNIMCI_msg_set_reference (UNIMCI_MSG *msg_inout ,const char* strReferenceToSet)
{
    int len;

    if (msg_inout->strHTMLReference)
        free (msg_inout->strHTMLReference);

    len = strlen (strReferenceToSet);
    msg_inout->strHTMLReference = (char*) malloc (sizeof(char) * (len+1));
    assert (msg_inout->strHTMLReference);
    if (strReferenceToSet)
        sprintf (msg_inout->strHTMLReference, "%s", strReferenceToSet);
    else
        sprintf (msg_inout->strHTMLReference, "");

    return 0;
}

/*========================================*/
/*          UNIMCI_msg_set_callinfo       */
/*========================================*/
int UNIMCI_msg_set_callinfo (UNIMCI_MSG *msg_inout ,const char* strCallInfoToSet)
{
    int len;

    if (msg_inout->strCallInfo)
        free (msg_inout->strCallInfo);

    len = strlen (strCallInfoToSet);
    msg_inout->strCallInfo = (char*) malloc (sizeof(char) * (len+1));
    assert (msg_inout->strCallInfo);
    if (strCallInfoToSet)
        sprintf (msg_inout->strCallInfo, "%s", strCallInfoToSet);
    else
        sprintf (msg_inout->strCallInfo, "");

    return 0;
}

/*========================================*/
/*          UNIMCI_msg_set_global         */
/*========================================*/
int UNIMCI_msg_set_global (UNIMCI_MSG *msg_inout ,int numParticipatingRanks ,const int* participatingRanks)
{
    int i;

    if (msg_inout->isGlobal && msg_inout->participatingRanks)
        free (msg_inout->participatingRanks);

    msg_inout->numParticipatingRanks = numParticipatingRanks;
    msg_inout->participatingRanks = (int*) malloc (sizeof(int) * numParticipatingRanks);
    assert (msg_inout->participatingRanks);

    for (i = 0; i < numParticipatingRanks; i++)
    {
        msg_inout->participatingRanks[i] = participatingRanks[i];
    }

    msg_inout->isGlobal = 1;

    return 0;
}

