/*
 * UniMCI (Universal MPI Correctness Interface)
 * Copyright (c) 2008-2009, ZIH, TU Dresden, Federal Republic of Germany
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file config.cxx
 * @brief Source for the UniMCI config application.
 * @details It provides tools that want to use UniMCI with information
 * about the configuration that is needed to compile with this
 * UniMCI installation and its back-end MPI correctness checker.
 *
 * Supported options:
 * @li --help            ~ prints the help
 * @li --version         ~ version of UniMCI (Major.Minor.BugFix)
 * @li --checker-version ~ version of the MPI checker (Major.Minor.BugFix)
 * @li --checker-name    ~ name of the used MPI checker
 * @li --ldflags         ~ linker flags
 * @li --cc              ~ the c compiler
 * @li --cflags          ~ all c flags used
 * @li --clibs           ~ with language specific annotation, e.g., for linux -lmylib instead of libmylib.a
 * @li --clib-dirs       ~ with -L/... for linux, for windows only the actual paths in quotes !
 * @li --cxx
 * @li --cxxflags
 * @li --cxxlibs
 * @li --cxxlib-dirs
 * @li --f77
 * @li --f77flags
 * @li --f77libs
 * @li --f77lib-dirs
 * @li --f90
 * @li --f90flags
 * @li --f90libs
 * @li --f90lib-dirs
 *
 * Multiple options may be specified at once, the output of all individual
 * options will be appended in the specified order.
 *
 * @author Tobias Hilbrich
 * @date 02.12.2008
 *
 * $ID$
 */

#include "unimci-utility.h"
#include "config.h"
#include <string>
#include <map>
#include <iostream>

//generate a map that matched each option to a print function
typedef void (*PrintFunction) ();
std::map<std::string, PrintFunction> option_commands;

//core functions
int prepare_option_commands ();
int parse_option (std::string option);

//print functions
void printHelp ();
void printVersion ();
void printCheckerVersion ();
void printCheckerName ();
void printLdflags ();
void printCc ();
void printCflags ();
void printClibs ();
void printClibdirs ();
void printCxx ();
void printCxxflags ();
void printCxxlibs ();
void printCxxlibdirs ();
void printF77 ();
void printF77flags ();
void printF77libs ();
void printF77libdirs ();
void printF90 ();
void printF90flags ();
void printF90libs ();
void printF90libdirs ();

//helper functions
void helperPrintOSSpecificList (const char * list);

//===========================
// main
//===========================
int main (int argc, char** argv)
{
    int ret = 0;
    std::string *argvs = new std::string[argc];

    //Convert all argv's to strings
    for (int i = 0; i < argc; i++)
    {
      argvs[i].assign (argv[i]);
    }

    //prepare the options map
    prepare_option_commands ();

    //Issue generation of output for each argument
    for (int i = 1; i < argc; i++)
    {
        //a space needed ?
        if (i != 1)
        {
            std::cout << " ";
        }

        if (parse_option (argvs[i]))
            ret = 1;
    }

    //No arguments given ? So print the Help !
    if (argc < 2)
        printHelp();

    //print a newline
    std::cout << std::endl;

    return ret;
}

//===========================
// prepare_option_commands
//===========================
int prepare_option_commands ()
{
    option_commands.clear ();
    option_commands.insert (std::make_pair ("--help"           ,printHelp));
    option_commands.insert (std::make_pair ("--version"        ,printVersion));
    option_commands.insert (std::make_pair ("--checker-version",printCheckerVersion));
    option_commands.insert (std::make_pair ("--checker-name"   ,printCheckerName));
    option_commands.insert (std::make_pair ("--ldflags"        ,printLdflags));
    option_commands.insert (std::make_pair ("--cc"             ,printCc));
    option_commands.insert (std::make_pair ("--cflags"         ,printCflags));
    option_commands.insert (std::make_pair ("--clibs"          ,printClibs));
    option_commands.insert (std::make_pair ("--clib-dirs"      ,printClibdirs));
    option_commands.insert (std::make_pair ("--cxx"            ,printCxx));
    option_commands.insert (std::make_pair ("--cxxflags"       ,printCxxflags));
    option_commands.insert (std::make_pair ("--cxxlibs"        ,printCxxlibs));
    option_commands.insert (std::make_pair ("--cxxlib-dirs"    ,printCxxlibdirs));
    option_commands.insert (std::make_pair ("--f77"            ,printF77));
    option_commands.insert (std::make_pair ("--f77flags"       ,printF77flags));
    option_commands.insert (std::make_pair ("--f77libs"        ,printF77libs));
    option_commands.insert (std::make_pair ("--f77lib-dirs"    ,printF77libdirs));
    option_commands.insert (std::make_pair ("--f90"            ,printF90));
    option_commands.insert (std::make_pair ("--f90flags"       ,printF90flags));
    option_commands.insert (std::make_pair ("--f90libs"        ,printF90libs));
    option_commands.insert (std::make_pair ("--f90lib-dirs"    ,printF90libdirs));

    return 0;
}

//===========================
// parse_option
//===========================
int parse_option (std::string option)
{
    std::string str;
    bool found_option = false;

    std::map<std::string, PrintFunction>::iterator i;
    for (i = option_commands.begin(); i != option_commands.end(); i++)
    {
        str = i->first;
        if ( option.compare(str) == 0 )
        {
            i->second ();
            found_option = true;
        }
    }

    if (!found_option)
    {
        std::cerr << "Error: Unknown option: " << option << std::endl;
        return 1;
    }

    return 0;
}

//===========================
// printHelp
//===========================
void printHelp ()
{
    std::cout
        << "Usage:"
        << std::endl
        << "unimci-config"
        << " [--help]"
        << " [--version]"
        << " [--checker-version]"
        << " [--checker-name]"
        << " [--ldflags]"
        << " [--cc]"
        << " [--cflags]"
        << " [--clibs]"
        << " [--clib-dirs]"
        << " [--cxx]"
        << " [--cxxflags]"
        << " [--cxxlibs]"
        << " [--cxxlib-dirs]"
        << " [--f77]"
        << " [--f77flags]"
        << " [--f77libs]"
        << " [--f77lib-dirs]"
        << " [--f90]"
        << " [--f90flags]"
        << " [--f90libs]"
        << " [--f90lib-dirs]"
        << std::endl
        << std::endl
        << "--help            : prints this help" << std::endl
        << "--version         : prints the UniMCI version" << std::endl
        << "--checker-version : prints the version of the used MPI checker" << std::endl
        << "--checker-name]   : prints the name of the used MPI checker" << std::endl
        << "--ldflags         : prints the linker flags for UniMCI+Checker" << std::endl
        << "--cc              : compiler for C" << std::endl
        << "--cflags          : UniMCI+Checker flags for C" << std::endl
        << "--clibs           : UniMCI+Checker libraries for C" << std::endl
        << "                    (OS specific: for linux \"-lmylib ...\", for windows \"mylib.lib ...\")" << std::endl
        << "--clib-dirs       : UniMCI+Checker library directories for C" << std::endl
        << "                    (OS specific: for linux \"-L/dir ...\", for windows \"\"C:/My Dir\" ...\")" << std::endl
        << "--cxx             : compiler for C++" << std::endl
        << "--cxxflags        : UniMCI+Checker flags for C++" << std::endl
        << "--cxxlibs         : UniMCI+Checker libraries for C++" << std::endl
        << "                    (OS specific: for linux \"-lmylib ...\", for windows \"mylib.lib ...\")" << std::endl
        << "--cxxlib-dirs     : UniMCI+Checker library directories for C++" << std::endl
        << "                    (OS specific: for linux \"-L/dir ...\", for windows \"\"C:/My Dir\" ...\")" << std::endl
        << "--f77             : compiler for Fortran77" << std::endl
        << "--f77flags        : UniMCI+Checker flags for Fortran77" << std::endl
        << "--f77libs         : UniMCI+Checker libraries for Fortran77" << std::endl
        << "                    (OS specific: for linux \"-lmylib ...\", for windows \"mylib.lib ...\")" << std::endl
        << "--f77lib-dirs     : UniMCI+Checker library directories for Fortran77" << std::endl
        << "                    (OS specific: for linux \"-L/dir ...\", for windows \"\"C:/My Dir\" ...\")" << std::endl
        << "--f90             : compiler for Fortran90" << std::endl
        << "--f90flags        : UniMCI+Checker flags for Fortran 90" << std::endl
        << "--f90libs         : UniMCI+Checker libraries for Fortran90" << std::endl
        << "                    (OS specific: for linux \"-lmylib ...\", for windows \"mylib.lib ...\")" << std::endl
        << "--f90lib-dirs     : UniMCI+Checker library directories for Fortran90" << std::endl
        << "                    (OS specific: for linux \"-L/dir ...\", for windows \"\"C:/My Dir\" ...\")" << std::endl
        << std::endl
        << std::endl;
}

//===========================
// printVersion
//===========================
void helperPrintOSSpecificList (const char * input, bool is_lib_dirs)
{
    std::string list = input;
    list.append (";");
    std::size_t pos;
    std::size_t last_pos = 0;
    std::size_t pos_temp;
    std::string elem;

    do
    {
        pos = list.find (';',last_pos);

        if (pos != list.npos)
        {
            //a space needed ?
            if (last_pos != 0)
            {
                std::cout << " ";
            }

            pos_temp = pos;
            elem.assign (list, last_pos, pos_temp - last_pos );

#ifdef _WIN32
            std::cout << "\"" << elem << "\"";
#else
            if (is_lib_dirs)
                std::cout << "-L" << elem;
            else
                std::cout << elem;
#endif

            //save as last pos
            last_pos = pos+1;
        }

    } while (pos != list.npos);
}

//===========================
// printVersion
//===========================
void printVersion ()
{
    std::cout << UNIMCI_VERSION;
}

//===========================
// printCheckerVersion
//===========================
void printCheckerVersion ()
{
    std::cout << CHECKER_VERSION;
}

//===========================
// printCheckerName
//===========================
void printCheckerName ()
{
    std::cout << USED_MPI_CHECKER;
}

//===========================
// printLdflags
//===========================
void printLdflags ()
{
    //Using the C LD flags, we should probably reduce to one LD flags ..
    std::cout << UNIMCI_MPI_C_LD_FLAGS;
}

//===========================
// printCc
//===========================
void printCc ()
{
    std::cout << UNIMCI_MPI_C_COMPILER;
}

//===========================
// printCflags
//===========================
void printCflags ()
{
    std::cout << UNIMCI_MPI_C_FLAGS;
}

//===========================
// printClibs
//===========================
void printClibs ()
{
    helperPrintOSSpecificList (UNIMCI_MPI_C_LIB_LIST, false);
}

//===========================
// printClibdirs
//===========================
void printClibdirs ()
{
    helperPrintOSSpecificList (UNIMCI_MPI_C_LIB_DIR_LIST, true);
}

//===========================
// printCxx
//===========================
void printCxx ()
{
    std::cout << UNIMCI_MPI_CXX_COMPILER;
}

//===========================
// printCxxflags
//===========================
void printCxxflags ()
{
    std::cout << UNIMCI_MPI_CXX_FLAGS;
}

//===========================
// printCxxlibs
//===========================
void printCxxlibs ()
{
    helperPrintOSSpecificList (UNIMCI_MPI_CXX_LIB_LIST, false);
}

//===========================
// printCxxlibdirs
//===========================
void printCxxlibdirs ()
{
    helperPrintOSSpecificList (UNIMCI_MPI_CXX_LIB_DIR_LIST, true);
}

//===========================
// printF77
//===========================
void printF77 ()
{
    std::cout << UNIMCI_MPI_F77_COMPILER;
}

//===========================
// printF77flags
//===========================
void printF77flags ()
{
    std::cout << UNIMCI_MPI_F77_FLAGS;
}

//===========================
// printF77libs
//===========================
void printF77libs ()
{
    helperPrintOSSpecificList (UNIMCI_MPI_F77_LIB_LIST, false);
}

//===========================
// printF77libdirs
//===========================
void printF77libdirs ()
{
    helperPrintOSSpecificList (UNIMCI_MPI_F77_LIB_DIR_LIST, true);
}

//===========================
// printF90
//===========================
void printF90 ()
{
    std::cout << UNIMCI_MPI_F90_COMPILER;
}

//===========================
// printF90flags
//===========================
void printF90flags ()
{
    std::cout << UNIMCI_MPI_F90_FLAGS;
}

//===========================
// printF90libs
//===========================
void printF90libs ()
{
    helperPrintOSSpecificList (UNIMCI_MPI_F90_LIB_LIST, false);
}

//===========================
// printF90libdirs
//===========================
void printF90libdirs ()
{
    helperPrintOSSpecificList (UNIMCI_MPI_F90_LIB_DIR_LIST, true);
}
