/*
 * UniMCI (Universal MPI Correctness Interface)
 * Copyright (c) 2008-2009, ZIH, TU Dresden, Federal Republic of Germany
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file unimci-core.h @brief Header for UniMCI core functions.
 *
 * @details The core function include function to retrieve and query
 *          messages, to set a call-back function, and to switch
 *          UniMCI on or off.
 *
 * @author Tobias Hilbrich
 * @date 03.12.2008
 */

#include "unimci-message.h"

#ifndef __UNIMCI_CORE_H
#define __UNIMCI_CORE_H

/**
 * @brief Enumerates the MPI binding languages.
 */
typedef enum
{
    UNIMCI_LANGUAGE_C = 0, /**< MPI C bindings are used. */
    UNIMCI_LANGUAGE_FORTRAN /**< MPI Fortran bindings are used. */
} UNIMCI_BINDING_LANGUAGE;

/**
 * @brief Type that describes the UniMCI call-back function.
 * @see UNIMCI_register_msg_callback
 */
typedef int (*UNIMCI_MSG_CALLBACK)(UNIMCI_MSG *msg);

#if defined(__cplusplus)
extern "C" {
#endif

/**
 * @brief Turns UniMCI and its used MPI correctness
 * checker on.
 * @details Starts as switched on.
 * @see UNIMCI_off
 */
void UNIMCI_on (void);

/**
 * @brief Turns UniMCI and its used MPI correctness
 * checker off.
 * @details Starts as switched on.
 * Use this call and UNIMCI_on to hide your own
 * MPI calls from UniMCI.
 * @see UNIMCI_on
 */
void UNIMCI_off (void);

/**
 * @brief Sets the currently used MPI binding language.
 * @details As all UniMCI calls are in C style it can't
 *          differentiate whether the Fortran or C bindings
 *          of MPI are used. Thus, the current binding language
 *          may be set with this call. The default is that
 *          the C language is used.
 * @see UNIMCI_BINDING_LANGUAGE
 * @param language the new binding language.
 */
void UNIMCI_set_binding_language (UNIMCI_BINDING_LANGUAGE language);

/**
 * @brief Registers the given function as a call-back that is used to
 * return messages that were generated asynchronously.
 * @details Must be called directly after UNIMCI_check_post__MPI_Init*.
 * Usage of this function is optional, if no call-back is given
 * all asynchronous messages can be retrieved with UNIMCI_pop_msg.
 * However, for some errors, e.g., deadlock, it may be
 * impossible to perform such a call.
 * If a call-back function is given it may be called by another
 * thread, so the user code may need extra mutual exclusion code.
 * @param callback the call-back function passed to UniMCI.
 * @return 0 if the call was successful, 1 otherwise.
 */
int UNIMCI_register_msg_callback (UNIMCI_MSG_CALLBACK callback);

/**
 * @brief Returns whether the MPI checker has a not yet
 * queried message.
 * @return 1 if a not yet returned message exists,
 *         0 otherwise.
 */
int UNIMCI_has_msg (void);

/**
 * @brief Returns number of outstanding messages.
 * @return number of outstanding messages.
 */
int UNIMCI_get_num_msgs (void);

/**
 * @brief Returns the first not yet queried message.
 *
 * @details Usage Example:
 *
@code
UNIMCI_MSG *msg = NULL;
UNIMCI_pop_msg (&msg);
//do something with the msg
UNIMCI_msg_free (&msg);
@endcode
 *
 * @param msg_record address of a pointer which will be
 *        set to contain the retrieved record.
 * @return 0 if the call was successfull,
 *         1 if an error occurred or no message is available.
 */
int UNIMCI_pop_msg (UNIMCI_MSG **msg_record);

/**
 * @brief Returns all messages.
 *
 * @code
 UNIMCI_MSG **msgs = NULL;
 int num_msgs, i;

 UNIMCI_get_msgs (&msgs, &num_msgs);
 for (i = 0; i < num_msgs; i++)
 {
    //do something with msgs[i]
    UNIMCI_msg_free (&(msgs[i]));
 }
 if (msgs)
     free (msgs);
 msgs = NULL;
 @endcode
 *
 * @param out_msgs Pre: address of a UNIMCI_MSG** pointer, Post: array with pointers to the individual messages.
 * @param out_num_msgs number of messages in the array, equals ::UNIMCI_get_num_msgs.
 */
int UNIMCI_get_msgs (UNIMCI_MSG ***out_msgs, int *out_num_msgs);

/**
 * @brief Returns the name of the back-end MPI checker.
 * @return string with the name of the MPI checker.
 */
const char* UNIMCI_get_checker_name (void);

/**
 * @brief Returns the version of the MPI checker.
 * @return string with the name of the MPI checker.
 */
const char* UNIMCI_get_checker_version (void);

#if defined(__cplusplus)
}
#endif

#endif /*__UNIMCI_CORE_H*/
