/*
 * UniMCI (Universal MPI Correctness Interface)
 * Copyright (c) 2008-2009, ZIH, TU Dresden, Federal Republic of Germany
 *
 * See the file LICENSE.txt in the package base directory for details
 */

/**
 * @file unimci-message.h
 * @brief Header file for the UniMCI message object.
 *
 * @details This header describes the message object that is used to
 * pass error messages between the master and the slave.
 * (Master is the tool that uses UniMCI to get MPI checking
 *  functionality, slave is the tool used to provide this)
 *
 * @author Tobias Hilbrich
 * @date 10.12.2009
 */

#ifndef __UNIMCI_MESSAGE_H
#define __UNIMCI_MESSAGE_H

#if defined(__cplusplus)
extern "C" {
#endif

/**
 * @brief Describes all types of messages.
 */
typedef enum
{
    UNIMCI_MSG_TYPE_INFO = 0, /**< Used for messages that only contain interesting information. */
    UNIMCI_MSG_TYPE_WARNING, /**< Used for MPI constructs that are dangerous or of bad style, but still legal. */
    UNIMCI_MSG_TYPE_ERROR /**< Used for actual violations to the MPI standard. */
} UNIMCI_MSG_TYPE;

/**
 * @brief Describes a message returned by an MPI checker.
 * @details Free it with UNIMCI_msg_free.
 * @see UNIMCI_msg_free
 *
 * @todo Missing: global messages lack the eventDescriptors
 *       of the other associated messages.
 * @todo Not captured is multi-threading, i.e., which thread
 *       of a process, and messages involving multiple threads
 *       of a single process.
 */
typedef struct
{
    /** The Type of the message, @see UNIMCI_MSG_TYPE. */
    UNIMCI_MSG_TYPE       msgType;


    char                  *strCallInfo, /**< String describing the MPI call. */
                          *strText, /**< String that contains the message text. */
                          *strHTMLReference; /**< String that contains the address of a HTML site with MPI standard information. */

    int                   isGlobal /**< True if this is a global message (involves more than just one rank). */,
                          numParticipatingRanks, /**< If 'isGlobal', number of ranks that are involved in the global message. */
                          *participatingRanks; /**< If 'isGlobal', ranks of the involved ranks. */

    unsigned long long    eventDescriptor; /**< The id that was given to the UniMCI check call that triggered this message. */
} UNIMCI_MSG;

/**
 * @brief Frees a message.
 * @param msg the message to be freed, is set to NULL.
 * @return 0 if the operation was successful, 1 otherwise.
 */
int UNIMCI_msg_free (UNIMCI_MSG **msg);

/**
 * @brief Creates a new and empty message.
 * @details All strings will be set to NULL and still
 * have to be initialized.
 * @param msg the message to be created.
 * @return 0 if the operation was successful, 1 otherwise.
 */
int UNIMCI_msg_new (UNIMCI_MSG **msg);

/**
 * @brief Clones an existing message.
 * @details Also clones all memory used by a message.
 * @param msg_in the message to clone.
 * @param msg_out the newly created message.
 * @return 0 if the operation was successful, 1 otherwise.
 */
int UNIMCI_msg_clone (UNIMCI_MSG *msg_in, UNIMCI_MSG **msg_out);

/**
 * @brief Sets the text of a message.
 * @details If msg_inout->strText is not NULL, it will be freed.
 * @param msg_inout the message for which the text is set.
 * @param strTextToSet the new text.
 * @return 0 if the operation was successful, 1 otherwise.
 */
int UNIMCI_msg_set_text (UNIMCI_MSG *msg_inout ,const char* strTextToSet);

/**
 * @brief Sets the HTML reference of a message.
 * @details If msg_inout->strHTMLReference is not NULL, it will be freed.
 * @param msg_inout the message for which the HTML reference is set.
 * @param strReferenceToSet the new HTML reference.
 * @return 0 if the operation was successful, 1 otherwise.
 */
int UNIMCI_msg_set_reference (UNIMCI_MSG *msg_inout ,const char* strReferenceToSet);

/**
 * @brief Sets the callinfo of a message.
 * @details If msg_inout->strCallInfo is not NULL, it will be freed.
 * @param msg_inout the message for which the call info is set.
 * @param strCallInfoToSet the new call info.
 * @return 0 if the operation was successful, 1 otherwise.
 */
int UNIMCI_msg_set_callinfo (UNIMCI_MSG *msg_inout ,const char* strCallInfoToSet);

/**
 * @brief Sets the given message as a global message.
 * @details If msg_inout->numParticipatingRanks is not NULL, it will be freed.
 * @param msg_inout the message which is set as global.
 * @param numParticipatingRanks the number of ranks that participate
 *        in the global message.
 * @param participatingRanks the individual ranks that participate in
 *        the global message, array with numParticipatingRanks entries.
 * @return 0 if the operation was successful, 1 otherwise.
 */
int UNIMCI_msg_set_global (UNIMCI_MSG *msg_inout ,int numParticipatingRanks ,const int* participatingRanks);

#if defined(__cplusplus)
}
#endif

#endif /*__UNIMCI_MESSAGE_H*/
